package com.nisum.offertransactionservice.controller;

import com.nisum.offertransactionservice.dto.EndOfSaleReq;
import com.nisum.offertransactionservice.dto.Item;
import com.nisum.offertransactionservice.dto.OfferTransactionRequest;
import com.nisum.offertransactionservice.dto.OfferTransactionResponse;
import com.nisum.offertransactionservice.model.*;
import com.nisum.offertransactionservice.service.EndOfSaleService;
import com.nisum.offertransactionservice.service.EndOfTransactionService;
import com.nisum.offertransactionservice.service.OfferCallingPEService;
import org.junit.Assert;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.*;
import org.mockito.junit.MockitoJUnitRunner;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockBean;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@RunWith(MockitoJUnitRunner.class)
public class OfferControllerTest {

    @InjectMocks
    private OfferTransactionController offerTransactionController = new OfferTransactionController();

    @Mock
    private OfferCallingPEService offerCallingPEService;

    @Mock
    private EndOfTransactionService endOfTransactionService;

    @Mock
    private EndOfSaleService endOfSaleService;

    @Before
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void getOfferTransactionResponseTest() throws InterruptedException {
        OfferTransactionRequest offerTransactionRequest = getOfferTransactionRequest();
        //Mockito.when(offerTransactionController.getOfferTransactionResponse(offerTransactionRequest)).thenReturn(retOfferTransRes());
        Mockito.when(offerCallingPEService.getDiscountedItemList(offerTransactionRequest)).thenReturn(retOfferTransRes());
        OfferTransactionResponse offerTransactionResponse = offerTransactionController.getOfferTransactionResponse(offerTransactionRequest);
        Assert.assertNotNull(offerTransactionResponse);
    }

    @Test
    public void endOfSaleTest() throws IOException {
        Mockito.when(endOfSaleService.putEndofSaleResInDb(getEndOfSaleReq())).thenReturn(UUID.randomUUID().toString());
        String uuid = offerTransactionController.endOfSale(getEndOfSaleReq());
        Assert.assertNotNull(uuid);
    }

    @Test
    public void endOfTransactionTest() {
        Mockito.when(endOfTransactionService.endOfTransaction(ArgumentMatchers.anyString())).thenReturn("End Of Transaction note from Store Producer");
        String res = offerTransactionController.endOfTransaction(UUID.randomUUID().toString());
        Assert.assertEquals(res, "End Of Transaction note from Store Producer");
    }

    private EndOfSaleReq getEndOfSaleReq() {
        EndOfSaleReq endOfSaleReq = new EndOfSaleReq();
        endOfSaleReq.setOfferTransactionResponse(retOfferTransRes());
        endOfSaleReq.setOffers(returnOffers());
        return endOfSaleReq;
    }

    public List<OfferLookup> returnOffers() {
        OfferLookup offerLookup1 = new OfferLookup(1L, "12", 23L, "02", "AND", "BOGO");
        OfferLookup offerLookup2 = new OfferLookup(2L, "13", 24L, "02", "OR", "BXGY");
        List<OfferLookup> offerLookups = new ArrayList<>();
        offerLookups.add(offerLookup1);
        offerLookups.add(offerLookup2);
        return offerLookups.stream().collect(Collectors.toList());
    }

    private OfferTransactionResponse retOfferTransRes() {
        OfferTransactionResponse offerTransactionResponse = new OfferTransactionResponse();
        offerTransactionResponse.setHhid("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 6.05);
        Item item2 = new Item("Thumpsup", "11", 12.32);
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionResponse.setDiscountedItemList(itemList);
        return offerTransactionResponse;
    }

    private OfferTransactionRequest getOfferTransactionRequest() {
        OfferTransactionRequest offerTransactionRequest = new OfferTransactionRequest();
        offerTransactionRequest.setHhId("123");
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 13.05);
        Item item2 = new Item("Thumpsup", "11", 14.05);
        itemList.add(item1);
        itemList.add(item2);
        offerTransactionRequest.setItemList(itemList);
        return offerTransactionRequest;
    }

}
